clc;clear;close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GECAD GECCO and CEC 2021 Competition: Evolutionary Computation in Uncertain Environments: A Smart Grid Application
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath('functions') %Necessary functions to run the algorithms (encrypted)
tTotalTime=tic; %lets track total computational time

Select_testbed=2;
%Testbed 1: Optimal bidding in local energy markets
%Testbed 2: Flexibility management of home appliances to support DSO requests

Select_Algorithm=-2;
% 00: HyDE-DF algorithm (test algorithm)
%-02: HC2RCEDUMDA 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load MH parameters (e.g., get MH parameters from DEparameters.m file)
switch Select_Algorithm
    case 0
        addpath('HyDE-DF')
        algorithm='HyDEDF'; %'The participants should include their algorithm here'
        DEparameters %Function defined by the participant
        No_solutions=deParameters.I_NP; %Notice that some algorithms are limited to one individual
        
    case -2
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %Your algorithm can be inserted here
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        addpath('HC2RCEDUMDAalg')
        algorithm='HC2RCEDUMDA'; %'The participants should include their algorithm here'
        HC2RCEDUMDAparameters %Function defined by the participant
        No_solutions=hc2RCEDUMDAparameters.I_NP; %Notice that some algorithms are limited to one individual     
        
    otherwise
        fprintf(1,'No algorithm selected\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load Data base
noRuns=20; %this can be changed but final results should be based on 20 trials
DB=Select_testbed;
% 1: Case study testbed 1
% 2: Case study testbed 2

[caseStudyData, DB_name]=callDatabase(DB);

%% Label of the algorithm and the case study
Tag.algorithm=algorithm;
Tag.DB=DB_name;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Set other parameters
otherParameters =setOtherParameters(caseStudyData,No_solutions,Select_testbed);
otherParameters.one=setOtherParameters(caseStudyData,1,Select_testbed);
% %% Set lower/upper bounds of variables
[lowerB,upperB] = setVariablesBounds(caseStudyData,otherParameters, Select_testbed);

hold on;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Call the MH for optimizationclear
ResDB=struc([]);
for iRuns=1:noRuns %Number of trails
    tOpt=tic;
    rand('state',sum(noRuns*100*clock))% ensure stochastic indpt trials
    otherParameters.iRuns=iRuns;
    
    fprintf('Run : %d .\n', iRuns);
    
    switch Select_Algorithm
        case 0
            [ResDB(iRuns).Fit_and_p, ...
                ResDB(iRuns).sol, ...
                ResDB(iRuns).fitVector]= ...
                HyDE(deParameters,caseStudyData,otherParameters,lowerB,upperB);
            
        case -2
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %Your algorithm can be inserted here
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            [ResDB(iRuns).Fit_and_p, ...
                ResDB(iRuns).sol, ...
                ResDB(iRuns).fitVector]= ...
                HC2RCEDUMDA(hc2RCEDUMDAparameters,caseStudyData,otherParameters,lowerB,upperB);
    end
    
    ResDB(iRuns).tOpt=toc(tOpt); % time of each trial
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %% Save the results and stats
    Save_results
end
tTotalTime=toc(tTotalTime); %Total time
%% End of MH Optimization

for j=1:noRuns
      lower_violations=ResDB(j).sol<lowerB;
      upper_violations=ResDB(j).sol>upperB;
      many_low(j)=sum(lower_violations);
      many_up(j)=sum(upper_violations);
      fprintf('lower_violations : %d .\n', many_low(j));
      fprintf('upper_violations : %d .\n', many_up(j));
end